@@ -20,6 +20,7 @@ import time
 from urllib import parse as urlparse
 
 import requests
+from requests import exceptions as req_exc
 from urllib3.exceptions import InsecureRequestWarning
 
 from sushy import exceptions
@@ -29,13 +30,23 @@ from sushy import utils
 LOG = logging.getLogger(__name__)
 
 
+_RETRYABLE_EXCEPTIONS = (
+    req_exc.ConnectionError,
+    req_exc.ConnectTimeout,
+    req_exc.ReadTimeout,
+    req_exc.Timeout,
+    req_exc.ChunkedEncodingError,
+)
+
+
 class Connector:
 
     def __init__(
             self, url, username=None, password=None, verify=True,
             response_callback=None, server_side_retries=0,
             server_side_retries_delay=0,
-            default_request_timeout=60):
+            default_request_timeout=60,
+            connect_timeout=None):
         self._url = url
         self._verify = verify
         self._session = requests.Session()
@@ -44,6 +55,10 @@ class Connector:
         self._server_side_retries = server_side_retries
         self._server_side_retries_delay = server_side_retries_delay
         self._default_request_timeout = default_request_timeout
+        # If connect_timeout is specified, use a tuple (connect, read) for
+        # requests timeout. This allows faster failure when a BMC is
+        # unreachable while still allowing longer read timeouts for slow BMCs.
+        self._connect_timeout = connect_timeout
 
         # NOTE(TheJulia): In order to help prevent recursive post operations
         # by allowing us to understand that we should stop authentication.
@@ -130,6 +145,10 @@ class Connector:
             server_side_retries_left = self._server_side_retries
 
         timeout = timeout or self._default_request_timeout
+        # If connect_timeout is configured, use a tuple (connect, read) for
+        # the requests timeout to allow faster failure on unreachable BMCs.
+        if self._connect_timeout is not None:
+            timeout = (self._connect_timeout, timeout)
 
         url = path if urlparse.urlparse(path).netloc else urlparse.urljoin(
             self._url, path)
@@ -162,21 +181,37 @@ class Connector:
                    'data': utils.sanitize(data),
                    'blocking': blocking, 'timeout': timeout,
                    'session': extra_session_req_kwargs})
-        try:
-            response = self._session.request(method, url, json=data,
-                                             headers=headers,
-                                             verify=self._verify,
-                                             timeout=timeout,
-                                             **extra_session_req_kwargs)
-        except requests.exceptions.RequestException as e:
-            # Capture any general exception by looking for the parent
-            # class of exceptions in the requests library.
-            # Specifically this will cover cases such as transport
-            # failures, connection timeouts, and encoding errors.
-            # Raising this as sushy ConnectionError allows users to
-            # understand something bad has happened, and to
-            # allow them to respond accordingly.
-            raise exceptions.ConnectionError(url=url, error=e)
+
+        retries = self._server_side_retries or 3
+        delay = self._server_side_retries_delay or 2
+
+        for attempt in range(retries):
+            try:
+                response = self._session.request(
+                    method, url, json=data,
+                    headers=headers,
+                    verify=self._verify,
+                    timeout=timeout,
+                    **extra_session_req_kwargs
+                )
+                break
+            except _RETRYABLE_EXCEPTIONS as e:
+                if attempt < retries - 1:
+                    LOG.warning(
+                        "Transient error during Redfish request to %s "
+                        "(attempt %d/%d): %s", url, attempt + 1, retries, e)
+                    time.sleep(delay)
+                else:
+                    raise exceptions.ConnectionError(url=url, error=e)
+            except requests.exceptions.RequestException as e:
+                # Capture any general exception by looking for the parent
+                # class of exceptions in the requests library.
+                # Specifically this will cover cases such as transport
+                # failures, connection timeouts, and encoding errors.
+                # Raising this as sushy ConnectionError allows users to
+                # understand something bad has happened, and to
+                # allow them to respond accordingly.
+                raise exceptions.ConnectionError(url=url, error=e)
 
         if self._response_callback:
             self._response_callback(response)
@@ -298,7 +333,13 @@ class Connector:
                 raise exceptions.ConnectionError(url=url, error=m)
 
             mon = TaskMonitor.from_response(self, response, path)
-            mon.wait(timeout)
+            # TaskMonitor.wait() expects a single number, not a tuple.
+            # If timeout is a tuple (connect, read), use the read timeout.
+            if isinstance(timeout, tuple):
+                wait_timeout = timeout[1]
+            else:
+                wait_timeout = timeout
+            mon.wait(wait_timeout)
             response = mon.response
             exceptions.raise_for_response(method, url, response)
 
@@ -370,7 +411,12 @@ class Connector:
         :raises: ConnectionError
         :raises: HTTPError
         """
-        if etag is not None:
+        # NOTE(janders) in case headers with empty-string eTag are passed in,
+        # disregard it.
+        if headers:
+            if headers.get('If-Match') == '':
+                del headers['If-Match']
+        if etag:
             if not headers:
                 headers = {}
             headers['If-Match'] = etag
