@@ -17,13 +17,12 @@ import abc
 import collections
 import copy
 import enum
+from importlib import resources
 import io
 import json
 import logging
 import zipfile
 
-import pkg_resources
-
 from sushy import exceptions
 from sushy.resources import constants
 from sushy.resources import oem
@@ -33,7 +32,7 @@ from sushy import utils
 LOG = logging.getLogger(__name__)
 
 
-class Field(object):
+class Field:
     """Definition for fields fetched from JSON."""
 
     def __init__(self, path, required=False, default=None,
@@ -104,7 +103,7 @@ class Field(object):
 
                 logging.warning(
                     'Applying default "%s" on required, but missing '
-                    'attribute "%s"' % (self._default, path))
+                    'attribute "%s"', self._default, path)
 
             # Do not run the adapter on the default value
             return self._default
@@ -140,7 +139,7 @@ class CompositeField(collections.abc.Mapping, Field, metaclass=abc.ABCMeta):
     """Base class for fields consisting of several sub-fields."""
 
     def __init__(self, *args, **kwargs):
-        super(CompositeField, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self._subfields = dict(_collect_fields(self))
 
     def _load(self, body, resource, nested_in=None):
@@ -152,7 +151,7 @@ class CompositeField(collections.abc.Mapping, Field, metaclass=abc.ABCMeta):
         :returns: a new object with sub-fields attached to it.
         """
         nested_in = (nested_in or []) + self._path
-        value = super(CompositeField, self)._load(body, resource)
+        value = super()._load(body, resource)
         if value is None:
             return None
 
@@ -186,7 +185,7 @@ class ListField(Field):
     """Base class for fields consisting of a list of several sub-fields."""
 
     def __init__(self, *args, **kwargs):
-        super(ListField, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self._subfields = dict(_collect_fields(self))
 
     def _load(self, body, resource, nested_in=None):
@@ -198,7 +197,7 @@ class ListField(Field):
         :returns: a new list object containing subfields.
         """
         nested_in = (nested_in or []) + self._path
-        values = super(ListField, self)._load(body, resource)
+        values = super()._load(body, resource)
         if values is None:
             return None
 
@@ -228,7 +227,7 @@ class DictionaryField(Field):
     """Base class for fields consisting of dictionary of several sub-fields."""
 
     def __init__(self, *args, **kwargs):
-        super(DictionaryField, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self._subfields = dict(_collect_fields(self))
 
     def _load(self, body, resource, nested_in=None):
@@ -240,7 +239,7 @@ class DictionaryField(Field):
         :returns: a new dictionary object containing subfields.
         """
         nested_in = (nested_in or []) + self._path
-        values = super(DictionaryField, self)._load(body, resource)
+        values = super()._load(body, resource)
         if values is None:
             return None
 
@@ -289,7 +288,7 @@ class MappedField(Field):
             raise TypeError("The mapping argument must be a mapping or "
                             "an enumeration")
 
-        super(MappedField, self).__init__(
+        super().__init__(
             field, required=required, default=default, adapter=adapter)
 
 
@@ -324,7 +323,7 @@ class MappedListField(Field):
                             "an enumeration")
 
         self._mapping_adapter = adapter
-        super(MappedListField, self).__init__(
+        super().__init__(
             field, required=required, default=default,
             adapter=lambda x: x)
 
@@ -337,7 +336,7 @@ class MappedListField(Field):
         :returns: a new list object containing the mapped values.
         """
         nested_in = (nested_in or []) + self._path
-        values = super(MappedListField, self)._load(body, resource)
+        values = super()._load(body, resource)
 
         if values is None:
             return
@@ -376,7 +375,7 @@ class MessageListField(ListField):
     """
 
 
-class FieldData(object):
+class FieldData:
     """Contains data to be used when constructing Fields"""
 
     def __init__(self, status_code, headers, json_doc):
@@ -401,7 +400,7 @@ class FieldData(object):
         return self._json_doc
 
 
-class AbstractDataReader(object, metaclass=abc.ABCMeta):
+class AbstractDataReader(metaclass=abc.ABCMeta):
 
     def set_connection(self, connector, path):
         """Sets mandatory connection parameters
@@ -486,9 +485,9 @@ class JsonPackagedFileReader(AbstractDataReader):
     def get_data(self):
         """Gets JSON file from packaged file denoted by path"""
 
-        with pkg_resources.resource_stream(self._resource_package_name,
-                                           self._path) as resource:
-            json_data = json.loads(resource.read().decode(encoding='utf-8'))
+        ref = resources.files(self._resource_package_name).joinpath(self._path)
+        with ref.open(encoding='utf-8') as fp:
+            json_data = json.load(fp)
             return FieldData(None, None, json_data)
 
 
@@ -507,7 +506,7 @@ def get_reader(connector, path, reader=None):
     return reader
 
 
-class ResourceBase(object, metaclass=abc.ABCMeta):
+class ResourceBase(metaclass=abc.ABCMeta):
 
     redfish_version = None
     """The Redfish version"""
@@ -689,15 +688,19 @@ class ResourceBase(object, metaclass=abc.ABCMeta):
         if not self._is_stale and not force:
             return
 
+        data_source = ""
         if json_doc:
             self._json = json_doc
+            data_source = "from expanded document"
         else:
             self._json = self._reader.get_data().json_doc
 
         attributes = self._parse_attributes(self._json)
-        LOG.debug('Received representation of %(type)s %(path)s: %(json)s',
+        LOG.debug('Received representation of %(type)s %(path)s%(source)s: '
+                  '%(json)s',
                   {'type': self.__class__.__name__,
                    'path': self._path,
+                   'source': data_source,
                    'json': (attributes if self._log_resource_body
                             else '<stripped>')})
         self._do_refresh(force)
@@ -844,7 +847,13 @@ class ResourceLinksBase(ResourceBase, metaclass=abc.ABCMeta):
 
         :returns: A list of ``_resource_type`` objects
         """
-        return [self.get_member(id_) for id_ in self.members_identities]
+        members = []
+        for id_ in self.members_identities:
+            try:
+                members.append(self.get_member(id_))
+            except exceptions.ResourceNotFoundError:
+                LOG.warning('Skipping missing collection member %s', id_)
+        return members
 
 
 class ResourceCollectionBase(ResourceLinksBase):
